#include <stdio.h>		//标准输入输出定义
#include <stdlib.h>		//标准函数库定义
#include <unistd.h>		//UNIX标准函数定义
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>		//文件控制定义
#include <termios.h>		//PPSIX终端控制定义 
#include <errno.h>		//错误号定义 

#include "CommSerial.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define READ_BUFF_SIZE		(1024 * 16)
#define SEND_BUFF_SIZE		(1024 * 1)


////////////////////////////// COM Functions //////////////////////////////
HANDLE COMMOpen(int portNo)
{
	if(portNo > 9 || portNo < 1)
		return INVALID_HANDLE_VALUE;
	char comName[15];
	::sprintf(comName, "/dev/ttyS%d", portNo);
	
	
	struct termios opt;
	
	HANDLE hCom = open(comName,O_RDWR);
	if(hCom == INVALID_HANDLE_VALUE)
	{
	    perror("open device");
	    return INVALID_HANDLE_VALUE;
	}
	printf("Open device success,waiting user input ...\n");
	
	//取得当前串口配置
	tcgetattr(hCom,&opt);
	tcflush(hCom,TCIOFLUSH);
	
	//设置波特率
	cfsetispeed(&opt,B9600);
	cfsetospeed(&opt,B9600);

	//设置数据位 -8位数据位
	opt.c_cflag &= ~CSIZE;
	opt.c_cflag != CS8;
	
	//设置校验位
	opt.c_cflag &= ~PARENB;
	opt.c_cflag != CS8;

	//设置停止位
	opt.c_cflag &= ~CSTOPB;

	//设置超时时间
	opt.c_cc[VTIME] = 150;
	opt.c_cc[VMIN] = 0;

	//设置写入设备
	if(0 != tcsetattr(hCom,TCSANOW,&opt))
	{
		perror("set baudrate");
		return 0;
	} 
	tcflush(hCom,TCIOFLUSH);
	return hCom;
}

void COMMClose(HANDLE hCom)
{
	if(hCom != INVALID_HANDLE_VALUE)
	{
		//::PurgeComm(hCom, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
		close(hCom);
	}
}

int	COMMSendData(HANDLE hCom, const char* data, int length)
{
   
	if(hCom == INVALID_HANDLE_VALUE)
		return COMM_ReturnError;
	int r = write(hCom,data,length);
	if(r > 0)
		return r;
	else
		return COMM_ReturnTimeOut;
}

int	COMMReceiveData(HANDLE hCom, char* data, int length)
{
	if(hCom == INVALID_HANDLE_VALUE)
		return COMM_ReturnError;
	
	int r= read(hCom, data, length);	
	if (r > 0)
		return r;
	else
		return COMM_ReturnTimeOut;
	
}

bool COMMConfigDCB(HANDLE hCom)
{
    return true;
}

bool COMMPurgeComm(HANDLE hCom)
{
  /*
	if(hCom != INVALID_HANDLE_VALUE && ::PurgeComm(hCom, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR))
		return true;
	else
		return false;*/
    return true;
}

bool COMMSetReadTimeOut(HANDLE hCom, unsigned int readTimeout)
{
    return true;
}

bool COMMSetWriteTimeOut(HANDLE hCom, unsigned int writeTimeout)
{
    return true;
}
//////////////////////////// General Functions ////////////////////////////
char GetCheckSum(char* buf, char len)
{
	char sum = 0;
	for(int i = 0; i < len; i++)
		sum = sum + buf[i];
	sum = (~sum) + 1;
	return sum;
}
